set(AIX 1)

set(CMAKE_STATIC_LIBRARY_PREFIX "lib")
set(CMAKE_STATIC_LIBRARY_SUFFIX ".a")
set(CMAKE_SHARED_LIBRARY_PREFIX "lib")
set(CMAKE_SHARED_LIBRARY_SUFFIX ".a")
set(CMAKE_SHARED_MODULE_PREFIX "lib")
set(CMAKE_SHARED_MODULE_SUFFIX ".so")
set(CMAKE_DL_LIBS "-lld")

set(CMAKE_MODULE_EXISTS 1)
####### TODO #######
#set(CMAKE_FIND_LIBRARY_SUFFIXES ".a" ".so")


# RPATH support on AIX is called libpath.  By default the runtime
# libpath is paths specified by -L followed by /usr/lib and /lib.  In
# order to prevent the -L paths from being used we must force use of
# -Wl,-blibpath:/usr/lib:/lib whether RPATH support is on or not.
# When our own RPATH is to be added it may be inserted before the
# "always" paths.
if(NOT DEFINED CMAKE_PLATFORM_REQUIRED_RUNTIME_PATH)
  set(CMAKE_PLATFORM_REQUIRED_RUNTIME_PATH /usr/lib /lib)
endif()

####### TODO #######
# Files named "libfoo.a" may actually be shared libraries.
#set_property(GLOBAL PROPERTY TARGET_ARCHIVES_MAY_BE_SHARED_LIBS 1)

# since .a can be a static or shared library on AIX, we can not do this.
# at some point if we wanted it, we would have to figure out if a .a is
# static or shared, then we could add this back:

# Initialize C link type selection flags.  These flags are used when
# building a shared library, shared module, or executable that links
# to other libraries to select whether to use the static or shared
# versions of the libraries.
#foreach(type SHARED_LIBRARY SHARED_MODULE EXE)
#  set(CMAKE_${type}_LINK_STATIC_C_FLAGS "-bstatic")
#  set(CMAKE_${type}_LINK_DYNAMIC_C_FLAGS "-bdynamic")
#endforeach()

# AIX can need to export symbols (typically for module).
# export_file is target name.
# export_obj are .o whose symbols are exported.
# export_path is where export file will be saved.
# ext is code export file, see man ld.
macro(ExportAIX export_file export_obj export_path ext)
  add_custom_command(TARGET ${export_file}
    PRE_LINK
    COMMAND echo "#! ${ext}" > ${export_path}
    COMMAND nm -BCg $<TARGET_OBJECTS:${export_obj}> | grep " [TDB] " | sed -e "s/.* //" | grep -v "\\\$" | sed -e "s/\^[.]//" | sort | uniq >> ${export_path}
    VERBATIM
  )
  target_link_options(${export_file}
    PRIVATE "-Wl,-bE:${export_path}")
  add_dependencies(${export_file} ${export_obj})
  set_property(DIRECTORY APPEND PROPERTY ADDITIONAL_MAKE_CLEAN_FILES ${export_path})
endmacro(ExportAIX)

macro(ImportAIX import_file export_path)
  target_link_options(${import_file}
    PRIVATE "-Wl,-bI:${export_path}")
endmacro(ImportAIX)

include(Platform/UnixPaths)
